"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getDeclarationTypeText_1 = require("./getDeclarationTypeText");
var getTypeText_1 = require("./getTypeText");
var nodeToString_1 = require("./nodeToString");
function getParameters(declaration, namespacesToInclude) {
    var parameters = getParameterDeclarations(declaration);
    if (!parameters) {
        var name = declaration.name ? nodeToString_1.nodeToString(declaration.name) : 'unknown';
        throw new Error("Missing declaration parameters for \"" + name + "\" in " + declaration.getSourceFile().fileName + " at line " + declaration.getStart());
    }
    return parameters.map(function (parameter) {
        var paramText = '';
        if (parameter.dotDotDotToken)
            paramText += '...';
        paramText += nodeToString_1.nodeToString(parameter.name);
        if (parameter.questionToken)
            paramText += '?';
        var type = parameter.type;
        if (type) {
            paramText += ': ' + getTypeText_1.getTypeText(type, namespacesToInclude);
        }
        var initializer = getDeclarationTypeText_1.getInitializer(parameter);
        if (initializer) {
            paramText += ' = ' + nodeToString_1.nodeToString(initializer);
        }
        return paramText.trim();
    });
}
exports.getParameters = getParameters;
function getParameterDeclarations(declaration) {
    return declaration.parameters;
}
//# sourceMappingURL=getParameters.js.map